PROGRAM EcosystemModel__LinearAnalysis_and_NTR_Version;
{ K.L. Cottingham - UW Madison Center For Limnology }

{ 6 Compartment Linear Model }
{ Small Fish as BG, not LMB; Chaob with Excr=Resp; No algal resp losses }
{ No chaoborus hypolimnetic excretions; algal growth as min(light,nutrients)*temp}
{ 100 time steps per day! }

{ last changed early December 1993
       (1) fixed units on some of the flux rates (see procedure) - 9 Dec 93
       (2) made consumption change each week as weight increases - 9 Dec 93
       (3) fixed Oak Ridge temperature function to ln(Q10) - 9 Dec 93
       (4) changed how I add across compartments (see memo) - 4 Dec 93
       (5) turned on saving the simulation to ASCII - 9 Dec 93
       (6) turned off paramout.dat and results.dat outfiles - 9 Dec 93
       (7) separate matlab and c program output files - 9 Dec 93
       (8) fixed ntr calculations - 9 Dec 93
       (9) took out old summary variable stuff - 9 Dec 93
       (10) corrected cmax : re-initialize between perturbation runs - 15 Dec 93
}

{ previously most recent changes made 11 March 1993 in the morning }

{ pulse perturbation on day 50 as done in pulseex2.pas }

{ *********************************************************************** }

USES
        Crt;

CONST
        NumberOfStepsPerDay = 100;
        WeekSize = 7;
        MaxWeeks = 25;                { maximum possible weeks to simulate }
        MaxDays = MaxWeeks * WeekSize;
        NStateVars = 12;              { number of state variables }
        MaxSize = 200;                { limit on array size for state variables }
        MatrixSize = 6;

        CA_SF = 0.182;                { bluegill CA for cmax bioenergetics }
        CB_SF = -0.274;               { bluegill CB for cmax bioenergetics }
        CA_LF = 0.33;                 { LMG CA for cmax bioenergetics }
        CB_LF = -0.325;               { LMB CB for cmax bioenergetics }

        Nperts = 10;

        PrintDefinitions = false;     { whether or not to define how the flags are set in the output }
        WriteOpeningMessage = false;  { whether or not to write a user-friendly message prior to running }

{ *********************************************************************** }

TYPE
{ last revised 27 April 1992 }

{ Interaction Terms }

         PredPrey = (sz_sa, lz_sa, lz_la, ch_sz,
                 sf_lz, sf_ch, sf_b, lf_ch, lf_sf, lf_b);

         PredPreyArray = ARRAY [sz_sa .. lf_b] OF REAL;

{ State Variables }

         SixVector = ARRAY [1..6] of REAL;

         SVVector = ARRAY [1..Nstatevars] of REAL;

         AStateVar = ARRAY [1..MaxDays] OF REAL;

         Names = RECORD
                 Ph, SmAlg, LarAlg, SmZ, LarZ, Chaob, SmF, LarF : REAL
                 END;

         EpiSVType = RECORD
                 P, SA, LA, SZ, CH, SF, LF : AStateVar
                 END;

         MetaSVType = RECORD
                 P, SA, LA, SZ : AStateVar
                 END;

         StateVariables = RECORD
                 epilimnion : EpiSVType;
                 metalimnion : MetaSVType;
                 day,
                 LZ : AStateVar
                 END;

         SummaryVariables = RECORD
                 Phosphorus, Algae, Zooplankton, Chaoborus, SmallFish, LargeFish : AStateVar;
                 END;

{ Migration Accounting }

         MigLocation = PACKED ARRAY [1..10] OF char;
                {E=epi; M=meta; L=littoral; H=hypo}

         CurrMigScheds = RECORD
                 where_LZ, where_CH, where_SF, where_LF : MigLocation;
                 END;

         Locations = RECORD
                 LZhere, CHhere, SFhere, LFhere : REAL;
                 END;

{ Input Schedules }

        DailyScheds = ARRAY [1..MaxDays,1..2] of REAL;

        WeeklyRec = RECORD
                  EpiTemp, EpiThick,
                  MetaTemp, MetaThick : REAL;
                  LZMig, CHMig, SFMig, LFMig : MigLocation
                  END;

        WeeklyScheds = ARRAY [1..MaxWeeks] of WeeklyRec;

        LoadType = ARRAY [1..10] of REAL;

        RespRec = ARRAY [1..5] of REAL;

{ Parameters }
        TempRecord = RECORD
                 Tmax, Topt, Q10 : REAL
                 END;

         MostlyChangingLayerParam = RECORD
                 Weight_SF, Weight_LF,
                 ActSink_SA, ActSink_LA,
                 TempEff_SA, TempEff_LA,
                 TempEff_SZ, TempEff_LZ, TempEff_CH, TempEff_SF, TempEff_LF,
                 RespRate_SF, RespRate_LF, RespRate_CH,
                 ZoopExcr,
                 SF_EgestParam, SF_ExcrParam,
                 Excretion_SF, Excretion_LF,
                 Growth_SF, Growth_LF,
                 P_up_by_SA, P_up_by_LA,
                 phi_SZ, phi_LZ, phi_CH, phi_SF, phi_LF : REAL;                   
                 END;

         PhysicalParamPlus = RECORD
                         Ir,                      {light}
                         PInput,                  {P input; epi only}
                         Ent,                     {entrainment}
                         flush,                   {flushing}
                         thick,                   {layer thickness}
                         temp : REAL              {layer temperature}
                         END;

         Layer = RECORD
                         phys : PhysicalParamPlus;
                         MCLP : MostlyChangingLayerParam;
                         END;

         StaticBioticParameters = RECORD
               CHRespRec : RespRec;
               SATempRec, LATempRec,
               SZTempRec, LZTempRec,
               CHTempRec, SFTempRec, LFTempRec, SFRespTempRec : TempRecord;  {Tmax, Topt, Q10 for each population}
               cmax_SZ, cmax_LZ, cmax_CH, cmax_SF, cmax_LF,
                                        {maximum consumptions}
               PtoBiom_SZ, PtoBiom_LZ, PtoBiom_CH, PtoBiom_SF, PtoBiom_LF, PtoBiom_B,
                                        {P to biomass ratios}
               Vmax_SA, Vmax_LA,        {maximum growth rates}
               diamSA, diamLA,          {algal sizes}
               lenSZ, lenLZ,            {zooplankton sizes}
               Knut_SA, Knut_LA,        {nutrient half-saturation constant}
               Klight_SA, Klight_LA,     {light half-saturation constant}
               sink_SA, sink_LA,        {algal sinking rates in m per day}
               chltoP_SA, chltoP_LA,    {algal chl to P ratios}
               GrowthRate_SF, GrowthRate_LF, {fish growth rates}
               ExcretionRate_SF, ExcretionRate_LF, {fish excretion rates}
               death_SZ, death_LZ,      {death rates of zoops in the metalimnion}
               death_SF, death_LF : REAL;{death rates of fishes in the epilimnion}
               w,                       {selectivities}
               AE,                      {assimilation efficiencies}
               egest,                   {egestion fraction}
               exc : PredPreyArray;     {excretion fractions}
               END;

{ Flux Stuff }

       Matrix = ARRAY [0..MatrixSize, 0..MatrixSize] of REAL;

       FluxRec = RECORD
               ZoopCons, ChaobCons,
               SFConsofZoops, SFConsofChaob, SFConsofLittoral,
               LFConsofChaob, LFConsofSF, LFConsofLittoral,  {record total; average over RK calls}
               ZoopExcr, ChaobExcr, SFExcr, LFExcr,  {record total; average over RK calls}
               AlgalSinking, AlgalPUptake,           {record total; average over RK calls}
               ZoopDeathInMeta : REAL;
               END;

{ ************************************************************** }

VAR
{Last modified 11 March 1993 }

        SBP : StaticBioticParameters;
        Epi, Meta : Layer;

        StateVars : StateVariables;
        SummaryVars : SummaryVariables;

        SummerFluxes : FluxRec;
        SummerMatrix : Matrix;
        NutTurnRates : ARRAY[1..10,1..2] OF REAL;
        ComptTurnRates : ARRAY[1..10,1..6,1..2] OF REAL;

        SixUnperturbedSummerMeans : SixVector;

        CurrentMigScheds : CurrMigScheds;

        WeeklySch : WeeklyScheds;        {weekly input schedule array}
        DailySch : DailyScheds;         {daily input schedule array}

        continue : boolean;

        NDaysLAdominEpi, NClearDaysinEpi, NBloomDaysinEpi, PerturbNumber,
        NDaysLAdominMeta, NClearDaysinMeta, NBloomDaysinMeta,
        SimNumber, NNutrLoads, PressVsPulse, i, PerturbCounter,
        NumberofWeekstoRun, ActualNumberofWeeks, UsersWeekSize, SaveInterval : INTEGER;

        epA, epW, SFpvalue, egest_SF, egest_LF,
        CurrentNutrLoad, PulseofNutrients,
        PerturbPercentage : REAL;

        PerturbArray,
        NutrLoadings : LoadType;

        userinfile, weeklyinfile, dailyinfile,
        SysOutfile1, SysOutfile2,
        MeanResults, MatrixFile, FluxFile, InputVectorFile,
        NutrRates : text;

{ ************************************************************** }

PROCEDURE Constants;
{ last modified 7 April 1992 }
BEGIN
       SFpvalue:= 0.5;

       egest_SF := 0.28;
       egest_LF := 0.28;
       epA := 0.017;                  { chlorophyll-specific light extinction coefficient }
       epW := 0.5;                    { light extinction coefficient for water }
END;

{ ************************************************************** }

PROCEDURE AssignNames;

{ last modified 9 April 1992 }
{ assigns a string name to each of the files used by the program; names could be included in the userinfile if desired }

BEGIN
       {assign(InitialParamOutfile, 'ParamOut.dat');}
        assign(weeklyinfile, 'WeeklyIn.dat');
        assign(dailyinfile, 'DailyIn.Dat');
       {assign(simulationresults, 'Results.Dat');}
       {assign(MeanResults, 'Means.Dat');}
        assign(Nutrrates,'noneqntr.dat');

       {rewrite(SimulationResults);}
       {rewrite(MeanResults);}
        rewrite(NutrRates);
END;

{ ************************************************************** }

PROCEDURE GetUserInput (VAR infile:text);

{Reads information from infile and stores it in the appropriate places within the program's memory.}
{Changes StateVars, SBP, epi, and meta by side effect}

{last modified 11 March 1993}

VAR
        i : INTEGER;
        blank : real;
BEGIN
         reset(infile);

         WITH StateVars DO
                 BEGIN
                 {Line 1: State Variables}
                 WITH epilimnion DO
                         read(infile, P[1], SA[1], LA[1], SZ[1]);
                 WITH metalimnion DO
                         read(infile, P[1], SA[1], LA[1], SZ[1]);
                 read(infile, LZ[1]);
                 WITH epilimnion DO
                         read(infile, CH[1], SF[1], LF[1]);
                 readln(infile);
                 END;

         {Line 2: Epilimnetic Parameters}
         WITH Epi.phys DO
                 readln(infile, flush, Ent);
         {Line 3: Metalimnetic Parameters}
         WITH Meta.phys DO
                 readln(infile, flush, Ent);

         WITH SBP DO
                 BEGIN
                 {Line 4: Small Algae}
                 readln(infile, vmax_SA, klight_SA, knut_SA, sink_SA, chltoP_SA);
                 {Line 5: Large Algae}
                 readln(infile, vmax_LA, klight_LA, knut_LA, sink_LA, chltoP_LA);

                 {Line 6: Small Zooplankton}
                 readln(infile, death_SZ, cmax_SZ, PtoBiom_SZ);
                 {Line 7: Large Zooplankton}
                 read(infile, death_LZ, cmax_LZ, PtoBiom_LZ);

                 {Line 8: Chaoborus AND Benthos}
                 readln(infile, cmax_CH, PtoBiom_CH);

                 WITH epi.MCLP DO
                      BEGIN
                      {Line 9: Small Fish}
                      readln(infile, death_SF, cmax_SF, PtoBiom_SF, GrowthRate_SF, Weight_SF);
                      {Line 10: Large Fish}
                      readln(infile, death_LF, cmax_LF, PtoBiom_LF, GrowthRate_LF, Weight_LF);
                      END;

                 ExcretionRate_SF := (1 - Egest_SF - GrowthRate_SF);
                 ExcretionRate_LF := (1 - Egest_LF - GrowthRate_LF);

                 {Lines 11-20: Predator-Prey}
                 readln(infile, AE[sz_sa], exc[sz_sa], w[sz_sa]);
                 readln(infile, AE[lz_sa], exc[lz_sa], w[lz_sa]);
                 readln(infile, AE[lz_la], exc[lz_la], w[lz_la]);
                 readln(infile, AE[ch_sz], exc[ch_sz], w[ch_sz]);
                 readln(infile, w[sf_lz]);
                 readln(infile, w[sf_ch]);
                 readln(infile, w[sf_b]);
                 readln(infile, w[lf_ch]);
                 readln(infile, w[lf_sf]);
                 readln(infile, w[lf_b]);

                 {Line 21: Run Specifics}
                 readln(infile, NumberOfWeekstoRun, UsersWeekSize, SaveInterval, PerturbPercentage, PerturbNumber);

                 {Lines 22 & 23: Temperature Factors}
                 WITH SATempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH LATempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH SZTempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH LZTempRec DO
                      readln(infile, Tmax, Topt, Q10);
                 WITH CHTempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH SFTempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH SFRespTempRec DO
                      read(infile, Tmax, Topt, Q10);
                 WITH LFTempRec DO
                      readln(infile, Tmax, Topt, Q10);

                 {Line 24 : Nutrient Loading Schedules}
                 read(infile, NNutrLoads);
                 FOR i := 1 to NNutrLoads DO
                     read(infile, NutrLoadings[i]);
                 readln(infile);

                 {Line 25 : Respiration Rates}
                 FOR i := 1 to 5 DO
                     read(infile, CHRespRec[i]);
                 readln(infile);

                 {Line 26 : Perturbation Sizes }
                 FOR i := 1 to Nperts DO
                     read(infile, PerturbArray[i]);
                 readln(infile);
                 END;

         close(infile);
END;

{ ***************************************************************** }

PROCEDURE ReadWeeklyScheds (VAR WeeklyInputSchedFile : TEXT) ;

{Last modified 14 January 1992 }
{Reads in the weekly input schedules from the file inputsch.dat; fills in the array.}

VAR
        i, j, index : integer;
        blank : char;
BEGIN

     reset(WeeklyInputSchedFile);
     index := 1;

  REPEAT
    WITH WeeklySch[index] DO
        BEGIN
        read(WeeklyInputSchedFile,EpiThick, EpiTemp, MetaThick, MetaTemp);

        read(WeeklyInputSchedFile, blank);
        WHILE (blank=' ') DO
              read(WeeklyInputSchedFile, blank);

        LZMig[1] := blank;

        FOR j := 2 to 10 DO
                 read(WeeklyInputSchedFile, LZMig[j]);
        read(WeeklyInputSchedFile,blank);
        FOR j := 1 to 10 DO
                 read(WeeklyInputSchedFile,CHMig[j]);
        read(WeeklyInputSchedFile,blank);
        FOR j := 1 to 10 DO
                 read(WeeklyInputSchedFile,SFMig[j]);
        read(WeeklyInputSchedFile,blank);
        FOR j := 1 to 10 DO
                 read(WeeklyInputSchedFile,LFMig[j]);
        readln(WeeklyInputSchedFile);
        index := index + 1;
        END;
     UNTIL (index>NumberofWeeksToRun+1) OR EOF(WeeklyInputSchedFile);

     close(WeeklyInputSchedFile);

     ActualNumberOfWeeks := index-1;

END;

{ ************************************************************** }

PROCEDURE ReadDailyScheds ( VAR DailyInputSchedFile : TEXT) ;

{Last modified 6 April 1992 }

{Reads in each of the daily input schedule arrays from the specified files}

VAR
     index, LastDay : integer;

BEGIN
     reset(DailyInputSchedFile);

     LastDay := (NumberofWeeksToRun + 1) * WeekSize;

     index := 1;
         REPEAT
                  readln(DailyInputSchedFile, DailySch[index,1]);
                  index := index + 1;
         UNTIL (index>LastDay) OR (eof(DailyInputSchedFile));


     close(DailyInputSchedFile);

END;   {ReadDailyScheds}

{ ************************************************************** }

PROCEDURE ZeroParameters;

{last modified 27 April 1992}

{     - initializes all unchanging parameters that aren't input by the users;
      - zeroes the flux elements;
      - zeroes out those parameters in the metalimnion that don't ever get used
}

BEGIN
         WITH meta.MCLP DO
                BEGIN
                phi_CH := 0.0;       phi_SF := 0.0;       phi_LF := 0.0;
                TempEff_CH := 0.0;   TempEff_SF := 0.0;   TempEff_LF := 0.0;
                RespRate_SF := 0.0;  RespRate_LF := 0.0;  RespRate_CH := 0.0;
                END;

         epi.phys.temp := 0.0;   epi.phys.thick := 0.0;   epi.phys.Ir := 0.0;
         meta.phys.temp := 0.0; meta.phys.thick := 0.0;  meta.phys.Ir := 0.0; meta.phys.PInput := 0.0;

END;

{ ******************************************************************* }

PROCEDURE ZeroFluxRec (VAR Fluxes : FluxRec);
{zeroes all elements of a flux record}
{last modified 9 December 1993}

VAR
      j,k:integer;

BEGIN
         FOR j := 1 to MatrixSize DO
             FOR k := 1 to MatrixSize DO
                 ComptTurnRates[PerturbCounter,j,k]:=0.0;

         WITH Fluxes DO
               BEGIN
               ZoopCons:= 0.0; ChaobCons:= 0.0;
               SFConsofZoops:= 0.0;   SFConsofChaob:= 0.0;   SFConsofLittoral:= 0.0;
               LFConsofChaob:= 0.0;   LFConsofSF:= 0.0;   LFConsofLittoral:= 0.0;    {record total; average over RK calls}
               ZoopExcr:=0.0; ChaobExcr:= 0.0;   SFExcr:= 0.0;   LFExcr:= 0.0;
               AlgalSinking:= 0.0;   AlgalPUptake:= 0.0;
               ZoopDeathInMeta:= 0.0;
               END;
END;

{ ************************************************************* }

PROCEDURE InitializeMatrix ( Var L : Matrix );
{ Initializes a matrix ; last modified 27 April 1992 }

VAR
      i, j : INTEGER;

BEGIN
     FOR i := 0 to MatrixSize DO
         FOR j := 0 to MatrixSize DO
              L[i,j] := 0.0
END;

{ ************************************************************* }

PROCEDURE PrintFluxes ( L : matrix; VAR Outfile : TEXT );
{ Prints a matrix to Outfile beginning with the row indicated by First }
{ last modified 27 April 1992 }

VAR
       i, j : INTEGER;

BEGIN
       FOR i := 1 to MatrixSize DO
             FOR j := 1 to MatrixSize DO
                    writeln(Outfile, L[i,j]:12:5, '   a',i,j);
       FOR i := 1 to MatrixSize DO
              writeln(Outfile, L[0,i]:12:5, '   a',0,i);
       FOR i := 1 to MatrixSize DO
              writeln(Outfile, L[i,0]:12:5, '   z',i);
END;


{ ************************************************************* }

PROCEDURE PrintMatrix ( L : matrix; VAR Outfile1, Outfile2 : TEXT );
{ Prints a matrix to Outfile beginning with the row indicated by First }
{ last modified 9 December 1993 }

VAR
       i, j : INTEGER;

BEGIN
       FOR i := 1 to MatrixSize DO
             BEGIN
             FOR j := 1 to MatrixSize DO
                   write(Outfile1, L[i,j]:10:6);
             writeln(Outfile1);
             END;

       FOR i := 1 to MatrixSize DO
             writeln(Outfile2, L[i,0]);

END;

{ ************************************************************* }

PROCEDURE PrintPhysicalParam ( physical : PhysicalParamPlus;
                             VAR outfile : text );

{last modified 22 November 1991}

{Helps condense the printing jobs by printing common physical features for both layers to outfile.
Called once for each layer.}

BEGIN
         WITH physical DO
                 BEGIN
                 writeln(outfile, '  Light = ', Ir:9:5);
                 writeln(outfile, '  P Input = ', PInput:9:5);
                 writeln(outfile, '  Entrainment = ', Ent:9:5);
                 writeln(outfile, '  Flushing = ', flush:9:5);
                 writeln(outfile, '  Thickness = ', thick:9:5);
                 writeln(outfile, '  Temperature = ', temp:9:5)
                 END

END;

{ ************************************************************* }

PROCEDURE PrintMigrations ( index : integer; VAR outfile : text );

{ last modified 14 January 1992 }
{ Prints migration pattern for the week referenced by index to outfile }

VAR
        i:integer;

BEGIN
        WITH WeeklySch[index] DO
             BEGIN
             writeln(outfile);
             write(outfile, 'Large Zoop Migrations: ');
             FOR i := 1 to 10 DO
                 write(outfile, LZMig[i]);
             writeln(outfile);
             write(outfile, 'Chaoborid  Migrations: ');
             FOR i := 1 to 10 DO
                 write(outfile, CHMig[i]);
             writeln(outfile);
             write(outfile, 'Small Fish Migrations: ');
             FOR i := 1 to 10 DO
                 write(outfile, SFMig[i]);
             writeln(outfile);
             write(outfile, 'Large Fish Migrations: ');
             FOR i := 1 to 10 DO
                 write(outfile, LFMig[i]);
             writeln(outfile);
             END;
END;
{ ************************************************************* }

PROCEDURE PrintSBP ( SBP : StaticBioticParameters;
                 VAR outfile : text);

{last modified 14 January 1992 }

{Prints contents of SBP to outfile.}

BEGIN

         WITH SBP DO
                 BEGIN
                 writeln(outfile, 'Algal Parameters:');  writeln(outfile);
                 writeln(outfile, '  maximum growth rates: SA = ',VMax_SA:9:5, '; LA = ', VMax_LA:9:5);
                 writeln(outfile, '  half saturation constants for nutrients : SA = ', KNut_SA:9:5, '; LA = ', Knut_LA:9:5);
                 writeln(outfile, '  half saturation constants for light : SA = ', Klight_SA:9:5, '; LA = ', Klight_LA:9:5);
                 writeln(outfile, '  sinking rates : SA = ', sink_SA:5:2, '; LA = ', sink_LA:5:2);
                 writeln(outfile, '  chlorophyll to P ratios : SA = ', chltoP_SA:9:5, '; LA = ', chltoP_LA:9:5);
                 writeln(outfile);
                 writeln(outfile,'Zooplankton Parameters:');  writeln(outfile);
                 writeln(outfile,'  death rates in metalimnion : SZ = ', death_SZ:9:5, '; LZ = ', death_LZ:9:5);
                 writeln(outfile,'  maximum cons. rates : SZ = ', cmax_SZ:9:5, '; LZ = ', cmax_LZ:9:5);
                 writeln(outfile,'  P to biomass ratios : SZ = ', PtoBiom_SZ:9:5, '; LZ = ', PtoBiom_LZ:9:5);
                 writeln(outfile);
                 writeln(outfile,'Chaoborus Parameters'); writeln(outfile);
                 writeln(outfile,'  maximum cons. rate = ', cmax_CH:9:5);
                 writeln(outfile,'  P to biomass ratio = ', PtoBiom_CH:9:5);
                 writeln(outfile);
                 writeln(outfile,'Fish Parameters');     writeln(outfile);
                 writeln(outfile,'  death rates in epilimnion :  SF = ', death_SF:5:2, '; LF = ', death_LF:5:2);
                 writeln(outfile,'  maximum cons. rates : SF = ', cmax_SF:9:5, '; LF = ', cmax_LF:9:5);
                 writeln(outfile,'  P to biomass ratios : SF = ', PtoBiom_SF:9:5, '; LF = ', PtoBiom_LF:9:5);
                 writeln(outfile);
                 writeln(outfile,'Selectivities; Assimilation and Excretion Rates:');  writeln(outfile);
                 writeln(outfile,'  SZ eating SA : w = ', w[sz_sa]:4:1, '; AE = ', AE[sz_sa]:4:1, '; Exc = ', Exc[sz_sa]:4:1);
                 writeln(outfile,'  LZ eating SA : w = ', w[lz_sa]:4:1, '; AE = ', AE[lz_sa]:4:1, '; Exc = ', Exc[lz_sa]:4:1);
                 writeln(outfile,'  LZ eating LA : w = ', w[lz_la]:4:1, '; AE = ', AE[lz_la]:4:1, '; Exc = ', Exc[lz_la]:4:1);
                 writeln(outfile,'  CH eating SZ : w = ', w[ch_sz]:4:1, '; AE = ', AE[ch_sz]:4:1, '; Exc = ', Exc[ch_sz]:4:1);
                 writeln(outfile,'  SF eating LZ : w = ', w[sf_lz]:4:1);
                 writeln(outfile,'  SF eating CH : w = ', w[sf_ch]:4:1);
                 writeln(outfile,'  SF eating  B : w = ', w[sf_b]:4:1);
                 writeln(outfile,'  LF eating CH : w = ', w[lf_ch]:4:1);
                 writeln(outfile,'  LF eating SF : w = ', w[lf_sf]:4:1);
                 writeln(outfile,'  LF eating  B : w = ', w[lf_b]:4:1);
                 writeln(outfile);
                 writeln(outfile,'Parameters for the Oak Ridge Temperature Function:');  writeln(outfile);

{add in algae!!}

                 write(outfile,'  SZ : Tmax = ', SZTempRec.tmax:5:1, '; Topt = ', SZTempRec.topt:5:1);
                      writeln(outfile, '; Q10 = ', SZTempRec.Q10:5:1);
                 write(outfile,'  LZ : Tmax = ', LZTempRec.tmax:5:1, '; Topt = ', LZTempRec.topt:5:1);
                      writeln(outfile, '; Q10 = ', LZTempRec.Q10:5:1);
                 write(outfile,'  CH : Tmax = ', CHTempRec.tmax:5:1, '; Topt = ', CHTempRec.topt:5:1);
                      writeln(outfile, '; Q10 = ', CHTempRec.Q10:5:1);
                 write(outfile,'  SF : Tmax = ', SFTempRec.tmax:5:1, '; Topt = ', SFTempRec.topt:5:1);
                      writeln(outfile, '; Q10 = ', SFTempRec.Q10:5:1);
                 write(outfile,'  LF : Tmax = ', LFTempRec.tmax:5:1, '; Topt = ', LFTempRec.topt:5:1);
                      writeln(outfile, '; Q10 = ', LFTempRec.Q10:5:1);
                 END;
END;

{ ************************************************************* }

PROCEDURE PrintInitialParamToFile (VAR outfile : text);

{last modified 14 January 1992}

{Calls up PrintSBP and PrintPhysicalParam; accesses both layers globally}

BEGIN
         rewrite(outfile);
         PrintSBP(SBP, Outfile);
         WITH Epi DO
                 BEGIN
                 writeln(outfile);
                 writeln(outfile, 'Epilimnetic Parameters');
                 writeln(outfile);
                 PrintPhysicalParam(phys, Outfile);
                 END;
         WITH Meta DO
                 BEGIN
                 writeln(outfile);
                 writeln(outfile, 'Metalimnetic Parameters');
                 writeln(outfile);
                 PrintPhysicalParam(phys, Outfile);
                 END;
         PrintMigrations(1, Outfile);
         close(outfile);
END;
{ *************************************************************************** }

FUNCTION FofT (NowTemp : REAL; TempRec : TempRecord ) : REAL;
{ last modified 9 December 1993 - ln(Q10) function }
{ previously modified 13 January 1992 }

{ Calculates the Oak Ridge Temperature Function for the parameters sent in.
Called by TempFxn.}

VAR
     first, second, Z, Y, V, X : REAL;
BEGIN

  WITH TempRec DO
     BEGIN
     Z := ln(Q10) * (Tmax - Topt);
     Y := ln(Q10) * (Tmax - Topt + 2);

     first := sqrt(1 + 40/Y);
     second := sqr(1 + first);

     X := (Z*Z * second) / 400;

     V := (Tmax - NowTemp) / (Tmax - Topt);

     IF NowTemp = Tmax THEN
           FofT := 0.0
     ELSE
           FofT := exp(X * ln(V)) * exp(X * (1-V));

     END;
END;

{ ************************************************************* }

PROCEDURE GetThisWeeksSched ( index : INTEGER );

{Last modified 6 April 1992}
{Changes a number of variables that are controlled by weekly input schedules  many by side effect  including those
within each layer's phys and the where array.}

CONST
        thisdebug = false;
VAR
        j : integer;
        TempEff_SFResp : Real;

FUNCTION xToy ( x, y : REAL ) : REAL;
BEGIN
     xToy := exp(y * ln(x));
END;

BEGIN
        WITH WeeklySch[index] DO
                 BEGIN
                 WITH Epi.phys DO
                 WITH Epi.MCLP DO
                 WITH SBP DO
                         BEGIN
                         thick := EpiThick;
                         ActSink_SA := SBP.sink_SA / thick; { convert sinking rates to per day}
                         ActSink_LA := SBP.sink_LA / thick;

                         temp := EpiTemp;

                         IF Index > 1 THEN
                               BEGIN
                                 Weight_SF := Weight_SF * exp(Growth_SF * 7);  {update fish weight for this week}
                                 Weight_LF := Weight_LF * exp(Growth_LF * 7);  {update fish weight for this week}
                               END;

                         cmax_SF := CA_SF * exp(CB_SF * ln(Weight_SF));
                         cmax_LF := CA_LF * exp(CB_LF * ln(Weight_LF));

                         TempEff_SA := FofT (temp, SBP.SATempRec);
                         TempEff_LA := FofT (temp, SBP.LATempRec);
                         TempEff_SZ := FofT (temp, SBP.SZTempRec);
                         TempEff_LZ := FofT (temp, SBP.LZTempRec);
                         TempEff_CH := FofT (temp, SBP.CHTempRec);
                         TempEff_SF := FofT (temp, SBP.SFTempRec);
                         TempEff_LF := FofT (temp, SBP.LFTempRec);

                         TempEff_SFResp := FofT (temp, SBP.SFRespTempRec);

                         SF_EgestParam := 0.158 * xToY(Temp, -0.22) * exp(0.631*SFpvalue);
                         SF_ExcrParam :=  0.0253 * xToY(Temp, 0.58) * exp(-0.299*SFpvalue);

                         RespRate_SF := 0.0154   * xToY(Weight_SF, -0.200) * TempEff_SFResp * 3.24;
                         RespRate_LF := 0.002835 * xToY(Weight_LF, -0.355) * exp(0.0811 * temp) * 3.24;
                         {the 3.24 converts g02/(g fish day) to /day -> 3240 cals/g 02, 1000 cals/g fish}

                         { Chaoborus Respiration Rate }
                         IF Temp <= 5.0 THEN
                                 RespRate_CH := CHRespRec[1];
                         IF (Temp>5.0) AND (Temp<=10.0) THEN
                                 RespRate_CH := CHRespRec[1] + (Temp-5.0)*(CHRespRec[2]-CHRespRec[1])/5.0;
                         IF (Temp>10.0) AND (Temp<=15.0) THEN
                                 RespRate_CH := CHRespRec[2] + (Temp-10.0)*(CHRespRec[3]-CHRespRec[2])/5.0;
                         IF (Temp>15.0) AND (Temp<=20.0) THEN
                                 RespRate_CH := CHRespRec[3] + (Temp-15.0)*(CHRespRec[4]-CHRespRec[3])/5.0;
                         IF (Temp>20.0) AND (Temp<=25.0) THEN
                                 RespRate_CH := CHRespRec[4] + (Temp-20.0)*(CHRespRec[5]-CHRespRec[4])/5.0;
                         IF Temp > 25.0 THEN
                                 RespRate_CH := CHRespRec[5];

                         END;

                 WITH Meta.phys DO
                         BEGIN
                         thick := MetaThick;
                         Meta.MCLP.ActSink_SA := SBP.sink_SA / thick;
                         Meta.MCLP.ActSink_LA := SBP.sink_LA / thick;

                         temp := MetaTemp;
                         Meta.MCLP.TempEff_SA := FofT (temp, SBP.SATempRec);
                         Meta.MCLP.TempEff_LA := FofT (temp, SBP.LATempRec);
                         Meta.MCLP.TempEff_SZ := FofT (temp, SBP.SZTempRec);
                         Meta.MCLP.TempEff_LZ := FofT (temp, SBP.LZTempRec);
                         END;

                 WITH CurrentMigScheds DO
                         FOR j := 1 TO 10 DO
                                 BEGIN
                                 where_lz[j] := LZMig[j];
                                 where_ch[j] := CHMig[j];
                                 where_sf[j] := SFMig[j];
                                 where_lf[j] := LFMig[j];
                                 END;
                 END;

END;  {Get This Week's Schedule}

{ *************************************************************************** }

PROCEDURE StartUpProgram;
{ last modified 28 April 1992 }
BEGIN
        Constants;
        ZeroParameters;
        GetUserInput(userinfile);
        AssignNames;
        ReadDailyScheds(dailyinfile);
        ReadWeeklyScheds(weeklyinfile);
        GetThisWeeksSched(1);
        Epi.phys.Ir := DailySch[1,1];
       { PrintInitialParamToFile(InitialParamOutfile); }
END;

{ ************************************************************** }

PROCEDURE StartUpNewSet;
{ modified for linear model, 9 December 1993 }
BEGIN
        CASE PerturbCounter OF
             1:BEGIN
               assign(MatrixFile, 'matrix_0.mat');
               assign(FluxFile, 'fluxes_0.toc');
               assign(InputVectorFile, 'inputs_0.mat');
               assign(SysOutfile1, 'systat_0.sep');
               assign(SysOutfile2, 'systat_0.agg');
               END;
             2:BEGIN
               assign(MatrixFile, 'matrix_1.mat');
               assign(FluxFile, 'fluxes_1.toc');
               assign(InputVectorFile, 'inputs_1.mat');
               assign(SysOutfile1, 'systat_1.sep');
               assign(SysOutfile2, 'systat_1.agg');
               END;
             3:BEGIN
               assign(MatrixFile, 'matrix_2.mat');
               assign(FluxFile, 'fluxes_2.toc');
               assign(InputVectorFile, 'inputs_2.mat');
               assign(SysOutfile1, 'systat_2.sep');
               assign(SysOutfile2, 'systat_2.agg');
               END;
             4:BEGIN
               assign(MatrixFile, 'matrix_3.mat');
               assign(FluxFile, 'fluxes_3.toc');
               assign(InputVectorFile, 'inputs_3.mat');
               assign(SysOutfile1, 'systat_3.sep');
               assign(SysOutfile2, 'systat_3.agg');
               END;
             5:BEGIN
               assign(MatrixFile, 'matrix_4.mat');
               assign(FluxFile, 'fluxes_4.toc');
               assign(InputVectorFile, 'inputs_4.mat');
               assign(SysOutfile1, 'systat_4.sep');
               assign(SysOutfile2, 'systat_4.agg');
               END;
             6:BEGIN
               assign(MatrixFile, 'matrix_5.mat');
               assign(FluxFile, 'fluxes_5.toc');
               assign(InputVectorFile, 'inputs_5.mat');
               assign(SysOutfile1, 'systat_5.sep');
               assign(SysOutfile2, 'systat_5.agg');
               END;
             7:BEGIN
               assign(MatrixFile, 'matrix_6.mat');
               assign(FluxFile, 'fluxes_6.toc');
               assign(InputVectorFile, 'inputs_6.mat');
               assign(SysOutfile1, 'systat_6.sep');
               assign(SysOutfile2, 'systat_6.agg');
               END;
             8:BEGIN
               assign(MatrixFile, 'matrix_7.mat');
               assign(FluxFile, 'fluxes_7.toc');
               assign(InputVectorFile, 'inputs_7.mat');
               assign(SysOutfile1, 'systat_7.sep');
               assign(SysOutfile2, 'systat_7.agg');
               END;
             9:BEGIN
               assign(MatrixFile, 'matrix_8.mat');
               assign(FluxFile, 'fluxes_8.toc');
               assign(InputVectorFile, 'inputs_8.mat');
               assign(SysOutfile1, 'systat_8.sep');
               assign(SysOutfile2, 'systat_8.agg');
               END;
            10:BEGIN
               assign(MatrixFile, 'matrix_9.mat');
               assign(FluxFile, 'fluxes_9.toc');
               assign(InputVectorFile, 'inputs_9.mat');
               assign(SysOutfile1, 'systat_9.sep');
               assign(SysOutfile2, 'systat_9.agg');
               END;
            END;  {Case Statement}

        rewrite(MatrixFile);
        rewrite(FluxFile);
        rewrite(SysOutfile1);
        rewrite(SysOutfile2);
        rewrite(InputVectorFile);
END;

{ ************************************************************** }

PROCEDURE StartUpNewRun;
{ last modified 28 April 1992 }
BEGIN
        ZeroFluxRec(SummerFluxes);
        Epi.MCLP.Weight_SF := 4;
        Epi.MCLP.Weight_LF := 160;
        GetThisWeeksSched(1);
        Epi.phys.Ir := DailySch[1,1];
END;


{ ************************************************************** }


FUNCTION GrowthEffect (size : CHAR;
         VAR exponent : REAL;
         light, thickness, temperature, TempEffect, Ph, SmAlg, LarAlg : REAL) : REAL;

{last modified 11 May 1992 }

{Computes the possible influences on algal growth (light, temperature, and P availability).

Returns the minimum of (light, nutrients) * temp effect.

Light availability derived by SRC & KLC;
temp function from Oak Ridge Temperature function - seen in Scavia (1980);
nutrient limitation is Michaelis-Menten uptake.

CHANGES exponent BY side effect!!!!!!!}

VAR
        TotalAlgChl, screening,
        lightlim, nutrlim, minlim : REAL;

BEGIN
        {1.  Compute light limitation}
        TotalAlgChl := SmAlg*SBP.chltoP_SA + LarAlg*SBP.chltoP_LA;
        screening := epA*TotalAlgChl + epW;
        exponent := -1 * screening * thickness;

        WITH SBP DO
                IF size='s' THEN
                         lightlim := ln((Klight_SA + light) / (Klight_SA + light*exp(exponent))) / (thickness*screening)
                ELSE
                         lightlim := ln((Klight_LA + light) / (Klight_LA + light*exp(exponent))) / (thickness*screening);

        {2.  Compute nutrient limitation}
        IF size='s' THEN
                nutrlim := Ph / (SBP.Knut_SA + Ph)
        ELSE
                nutrlim := Ph / (SBP.Knut_LA + Ph);

        {3.  Compute the minimum factor}
        minlim := nutrlim;
        IF minlim > lightlim THEN
                minlim := lightlim;

        {5.  Return the growth effect.}
        GrowthEffect := minlim*TempEffect;

END;  { FUNCTION growthlim }


{ *************************************************************************** }

PROCEDURE AreMigratorsInEpilimnion ( timestep : INTEGER;
                                  CurrentMigScheds : CurrMigScheds;
                                  VAR MigLocations : Locations );

{ last modified 4 December 1992 }
{ sees whether LZ, CH, SF, LF are in the epilimnion in this time step;
  sets variables in the record EpiMigLocations accordingly }

VAR
        EffectiveTimeStep : INTEGER;

BEGIN
        EffectiveTimeStep := (TimeStep DIV 10) + 1;
        IF EffectiveTimeStep > 10 THEN
                 EffectiveTimeStep := 10;
        WITH MigLocations DO
        WITH CurrentMigScheds DO
                 BEGIN
                 LZhere := 0.0;   CHhere := 0.0;   SFhere := 0.0;   LFhere := 0.0;
                 
                 IF (where_LZ[EffectiveTimeStep] = 'E') THEN
                         LZhere := 1.0;
                 IF (where_CH[EffectiveTimestep] = 'E') THEN
                         CHhere := 1.0;
                 IF (where_SF[EffectiveTimestep] = 'E') THEN
                         SFhere := 1.0;
                 IF (where_LF[EffectiveTimeStep] = 'E') THEN
                         LFhere := 1.0;
                 END
END;

{ ************************************************************** }

PROCEDURE AreMigratorsInMetalimnion ( timestep : INTEGER;
                                  CurrentMigScheds : CurrMigScheds;
                                  VAR MigLocations : Locations );

{ last modified 4 December 1992 }
{ sees whether LZ are in the metalimnion in this time step;
  sets variables in the record EpiMigLocations accordingly }

VAR
        EffectiveTimeStep : INTEGER;

BEGIN
        EffectiveTimeStep := (TimeStep DIV 10) + 1;
        IF EffectiveTimeStep > 10 THEN
                 EffectiveTimeStep := 10;
        WITH MigLocations DO
        WITH CurrentMigScheds DO
                 BEGIN
                 LZhere := 0.0;   CHhere := 0.0;   SFhere := 0.0;   LFhere := 0.0;
                 
                 IF (where_LZ[EffectiveTimestep] = 'M') THEN
                         LZhere := 1.0;
                 END
END;


{ ************************************************************** }

PROCEDURE ConvertVectortoNames (currvals : SVvector;
          EpiMigLocations, MetaMigLocations : Locations;
          isthisepi : boolean;
          VAR SVbyName : Names);

{Last modified 29 December 1991}
{Converts vector of state variable values into "word names" and accounts for where migrating populations are at the
time.}

BEGIN
    WITH SVbyName DO
        IF isthisepi THEN
                 WITH EpiMigLocations DO
                         BEGIN
                         Ph := currvals[1];  SmAlg := currvals[2];  LarAlg := currvals[3];
                         SmZ := currvals[4];  LarZ := LZhere*currvals[9];
                         Chaob := CHhere*currvals[10];  SmF := SFhere*currvals[11];  LarF := LFhere*currvals[12];
                         END
        ELSE
                 WITH MetaMigLocations DO
                         BEGIN
                         Ph := currvals[5];  SmAlg := currvals[6];  LarAlg := currvals[7];  SmZ := currvals[8];
                         LarZ := LZhere*currvals[9];
                         Chaob := 0.0;  SmF := 0.0;  LarF := 0.0;
                         END
END;

{ *********************************************************************** }

PROCEDURE ConvertFromStateVariableRecordtoVector (index : INTEGER;
        VAR currvals : SVvector);

{Converts from one time in the state variable arrays (as indicated by index) to a vector}

{Last modified 14 January 1991}

CONST
        thisdebug = false;
VAR
        i : integer;
BEGIN
        WITH StateVars DO
                BEGIN
                WITH epilimnion DO
                         BEGIN
                         currvals[1] := P[index];
                         currvals[2] := SA[index];        currvals[3] := LA[index];
                         currvals[4] := SZ[index];        currvals[10] := CH[index];
                         currvals[11] := SF[index];       currvals[12] := LF[index];
                         END;
                WITH metalimnion DO
                         BEGIN
                         currvals[5] := P[index];
                         currvals[6] := SA[index];        currvals[7] := LA[index];
                         currvals[8] := SZ[index];        
                         END;
                currvals[9] := LZ[index];
                END;

END;

{ *************************************************************************** }

PROCEDURE ComputePhi (MigPlaces : Locations;
          SVNames : Names;
          IsThisEpi : Boolean;
          VAR MCLP : MostlyChangingLayerParam );

{last modified 22 January 1992}

{Computes the denominators for type II functional responses for each consumer link in the food web, then multiplies
by the temperature functions as well. The ??here variables denote whether or not that particular population is present in
the layer under consideration; using them insures that phi_?? gets set to zero when the population is elsewhere.
Treats fish according to the minimum ration protocol.}

VAR
        SZadder, LZadder, CHadder, SFadder, LFadder : REAL;

BEGIN
        WITH SVNames DO
             BEGIN
             SZadder := SmZ;  LZadder := LarZ;   CHadder := Chaob;
             SFadder := SmF;  LFadder := LarF;
             END;

        WITH MCLP DO
        WITH SBP DO
            WITH SVNames DO
            WITH MigPlaces DO
                         BEGIN
                         phi_LZ := 0.0;  phi_CH := 0.0;   phi_SF := 0.0;   phi_LF := 0.0;

                         phi_SZ := TempEff_SZ * cmax_sz / (SZadder + w[sz_sa]*SmAlg);

                         IF LZhere=1.0 THEN
                               phi_LZ := LZhere * TempEff_LZ * cmax_lz / (LZadder + w[lz_sa]*SmAlg + w[lz_la]*LarAlg);
                         IF CHhere=1.0 THEN
                               phi_CH := CHhere * TempEff_CH * cmax_ch / (CHadder + w[ch_sz]*SmZ);

                         IF SFhere=1.0 THEN
                               phi_SF := SFhere * TempEff_SF * cmax_sf / (SFadder + w[sf_lz]*LarZ + w[sf_ch]*Chaob);

                         IF LFhere=1.0 THEN
                               phi_LF := LFhere * TempEff_LF * cmax_lf / (LFadder + w[lf_sf]*SmF + w[lf_ch]*Chaob);
                         END;

END;  { PROCEDURE ComputePhi }

{ ************************************************************* }



PROCEDURE ComputeExcretion ( VAR MCLP : MostlyChangingLayerParam;
                             SVNames : Names;
                             IsthisEpi : Boolean;
                             SBP : StaticBioticParameters );

{last modified 9 April 1992}

{Computes the excretion into the phosphorus compartment at a given time step, minus any excretion due to feeding
on benthic sources.}

CONST
        thisdebug = false;
VAR
        exSZ, exLZ : REAL;
BEGIN
        WITH SBP DO
        WITH MCLP DO
        WITH SVnames DO
             BEGIN
             exSZ := 0.0;   exLZ := 0.0;

             exSZ := (exc[sz_sa]*w[sz_sa]*SmAlg) * phi_SZ * SmZ;
             exLZ := (exc[lz_sa]*w[lz_sa]*SmAlg + exc[lz_la]*w[lz_la]*LarAlg) * phi_lz * LarZ;

             ZoopExcr := exSZ + exLZ;

             END

END;  {PROCEDURE ComputeExcretion}

{ ************************************************************* }

PROCEDURE FeedDaSmallFish (LarZ, Chaob, SmF : REAL;
                 VAR Growth, Excretion : REAL;
                 VAR F : FluxRec);          {flux matrix}

{ last modified 27 April 1992}
{ Derivation of respiration equations by Xi He; Xi & Daniel instrumental to getting this procedure going.}

CONST
        thisdebug = true;
VAR
        TotalConsumption, ModelPConsumption :  REAL;
BEGIN

    Growth := 0.0;         Excretion := 0.0;

    WITH SBP DO
    WITH epi.MCLP DO
            BEGIN
            F.SFConsOfZoops := phi_SF * w[sf_lz] * LarZ * SmF;
            F.SFConsofChaob := phi_SF * w[sf_ch] * Chaob * SmF;

            ModelPConsumption := phi_SF * (w[sf_lz] * LarZ + w[sf_ch] * Chaob);

            TotalConsumption := (GrowthRate_SF + RespRate_SF) /
                   (0.828 - 0.828*SF_EgestParam - SF_ExcrParam + SF_EgestParam*SF_ExcrParam);

            IF ModelPConsumption > TotalConsumption THEN
                   TotalConsumption := ModelPConsumption;

            F.SFConsofLittoral := (TotalConsumption - ModelPConsumption) * SmF;

            Growth := GrowthRate_SF;
            Excretion := TotalConsumption - GrowthRate_SF - Egest_SF*TotalConsumption;

            F.SFExcr := Excretion*SmF;

           END;
 END;

{ ************************************************************* }

PROCEDURE FeedDaLargeFish (Chaob, SmF, LarF : REAL;
                 VAR Growth, Excretion : REAL;
                 VAR F : FluxRec);

{ last modified 27 April 1992}
{ Derivation of respiration equations by Xi He; Xi & Daniel instrumental to getting this procedure going.}

CONST
        thisdebug = false;
VAR
        ModelPConsPerDay, TotalConsumption :  REAL;
BEGIN
    Growth := 0.0;         Excretion := 0.0;

    WITH SBP DO
    WITH epi.MCLP DO
            BEGIN
            F.LFConsOfSF := phi_SF * w[lf_sf] * SmF * LarF;
            F.LFConsofChaob := phi_LF * w[lf_ch] * Chaob * LarF;

            ModelPConsPerDay := (w[lf_sf]*SmF) * (w[lf_ch]*Chaob) * phi_LF;
            TotalConsumption := (GrowthRate_LF + RespRate_LF) * 1.45;

            IF ModelPConsPerDay > TotalConsumption THEN
                   TotalConsumption := ModelPConsPerDay;

            F.LFConsofLittoral := (TotalConsumption - ModelPConsPerDay) * LarF;

            Growth := GrowthRate_LF;
            Excretion := TotalConsumption - GrowthRate_LF - Egest_LF*TotalConsumption;

            F.LFExcr := Excretion*LarF;

            END;

END;

{ ************************************************************* }

PROCEDURE PreDerivs ( currvals : SVvector; {vector of current state var values}
                     EpiMigLocations, MetaMigLocations : Locations;
                     isthisepi : BOOLEAN;
                     VAR L : Layer;
                     VAR F : FluxRec);

{Computes the changing values and parameters needed to compute the derivatives in each time step.
 Changes meta.phys.Ir by side effect. Called once for each layer. }

CONST
        thisdebug = false;
VAR
        SVNames : Names;
        BPEaten, PRecycled, exponent : REAL;
        RightMigLocations : Locations;
BEGIN
        ConvertVectortoNames (currvals, EpiMigLocations, MetaMigLocations, isthisepi, SVNames);

        WITH L DO
             WITH phys DO
             WITH MCLP DO
             WITH SVNames DO
                BEGIN

                {1.  Compute algal uptake of phosphorus & save as flux rate}
                P_up_by_SA := SBP.Vmax_SA * GrowthEffect('s',exponent,Ir,thick,temp,TempEff_SA,Ph,SmAlg,LarAlg);
                P_up_by_LA := SBP.Vmax_LA * GrowthEffect('l',exponent,Ir,thick,temp,TempEff_LA,Ph,SmAlg,LarAlg);

                F.AlgalPUptake := F.AlgalPUptake + P_up_by_SA*SmAlg + P_up_by_LA*LarAlg; {sum between epi and meta}

                {2.  Compute metalimnion light if this is the epilimnion}
                IF isthisepi THEN
                         IF exponent>-100 THEN
                                 meta.phys.Ir := epi.phys.Ir * exp(exponent)
                         ELSE
                                 meta.phys.Ir := 0.0;

                {3.  Compute Phi}
                IF isthisepi THEN
                         ComputePhi(EpiMigLocations, SVNames, true, MCLP)
                ELSE
                         ComputePhi(MetaMigLocations, SVNames, false, MCLP);

                {4.  Compute excretion}  
                ComputeExcretion(MCLP, SVNames, isthisepi, SBP);

                {5.  Feed the fish "their benthos"}
                IF isthisepi THEN
                         BEGIN
                         IF (EpiMigLocations.SFhere=1.0) THEN
                                  FeedDaSmallFish(SVNames.LarZ, SVNames.Chaob, SVNames.SmF, Growth_SF, Excretion_SF, F);
                         IF (EpiMigLocations.LFhere=1.0) THEN
                                  FeedDaLargeFish(SVNames.Chaob, SVNames.SmF, SVNames.LarF, Growth_LF, Excretion_LF, F);
                         END;
                END

END;  {PROCEDURE PreDerivs}

{ *************************************************************************** }

PROCEDURE Derivs ( now : SVvector; {vector of current state variable values}
                 TimeStep : Integer;
                 VAR delta : SVvector; {vector of derivatives}
                 VAR F : FluxRec);     {flux matrix currently being filled}

{ Calculates the derivatives & selected flux rates}
{ Calls prederivs; accesses SBP }

VAR
        ChaobExcr, plus, minus : REAL;
        EpiMigLocations, MetaMigLocations : Locations;

BEGIN
        WITH SBP DO
            BEGIN
            AreMigratorsinEpilimnion(timestep, CurrentMigScheds, EpiMigLocations);
            AreMigratorsinMetalimnion(timestep, CurrentMigScheds, MetaMigLocations);

            { epilimnion }
            prederivs ( now, EpiMigLocations, MetaMigLocations, true, epi, F);
            WITH Epi DO
                WITH phys DO
                WITH MCLP DO
                    BEGIN
                    IF (EpiMigLocations.CHhere=1.0) THEN
                         ChaobExcr := RespRate_CH * now[10]
                    ELSE
                         ChaobExcr := 0.0;
                    plus := PInput + Ent + ZoopExcr + ChaobExcr + Excretion_SF + Excretion_LF;
                    F.ZoopExcr := ZoopExcr;
                    minus := (P_up_by_SA*now[2] + P_up_by_LA*now[3]);
                    delta[1] := plus - minus;

                    plus := P_up_by_SA;
                    minus := ActSink_SA + (w[sz_sa]*phi_SZ*now[4]) + (w[lz_sa]*phi_LZ*now[9]);
                    delta[2] := (plus-minus) * now[2];

                    plus := P_up_by_LA;
                    minus := ActSink_LA + (w[lz_la]*phi_LZ*now[9]);
                    delta[3] := (plus - minus) * now[3];

                    plus := AE[sz_sa]*w[sz_sa]*phi_SZ*now[2];
                    minus := w[ch_sz]*phi_ch*now[10];
                    delta[4] := (plus - minus) * now[4];

                    F.ZoopCons := w[sz_sa]*phi_SZ*now[2]*now[4];

                    IF (EpiMigLocations.LZhere=1.0) THEN
                        BEGIN
                        plus := (AE[lz_sa]*w[lz_sa]*now[2] + AE[lz_la]*w[lz_la]*now[3]) * phi_LZ;

                        F.ZoopCons := F.ZoopCons + (w[lz_sa]*now[2] + w[lz_la]*now[3])*phi_LZ*now[9];

                        minus := w[sf_lz]*phi_SF*now[11];
                        delta[9] := (plus - minus) * now[9]
                        END;

                    IF (EpiMigLocations.CHhere=1.0) THEN
                        BEGIN
                        plus := (AE[ch_sz]*w[ch_sz]*phi_CH*now[4]);
                        F.ChaobCons := w[ch_sz]*phi_CH*now[4]*now[10];
                        minus := RespRate_CH + w[sf_ch]*phi_SF*now[11] + w[lf_ch]*phi_LF*now[12];
                        delta[10] := (plus - minus) * now[10];
                        F.ChaobExcr := RespRate_CH * now[10];
                        END
                    ELSE
                        delta[10] := 0.0;

                    IF (EpiMigLocations.SFhere=1.0) THEN
                        BEGIN
                        plus := Growth_SF;
                        minus := (w[lf_sf]*phi_LF*now[12]) + Death_SF;
                        delta[11] := (plus - minus) * now[11];
                        END
                    ELSE
                        delta[11] := 0.0;

                    IF (EpiMigLocations.LFhere=1.0) THEN
                        BEGIN
                        plus := Growth_LF;
                        minus := death_LF;
                        delta[12] := (plus - minus) * now[12];
                        END
                    ELSE
                        delta[12] := 0.0;
                END;

                { metalimnion }
                prederivs( now, EpiMigLocations, MetaMigLocations, false, meta, F);
                WITH Meta DO
                     WITH phys DO
                     WITH MCLP DO
                             BEGIN
                             plus := Ent + ZoopExcr;
                             minus := Epi.phys.Ent + (P_up_by_SA*now[6] + P_up_by_LA*now[7]);
                             delta[5] := plus - minus;

                             plus := P_up_by_SA*now[6] + epi.MCLP.ActSink_SA*now[2];
                             minus := meta.MCLP.ActSink_SA + w[sz_sa]*phi_SZ*now[8] + w[lz_sa]*phi_LZ*now[9];
                             delta[6] := plus - minus*now[6];

                             F.AlgalSinking := ActSink_SA*now[6];

                             plus := P_up_by_LA*now[7] + epi.MCLP.ActSink_LA*now[3];
                             minus := meta.MCLP.ActSink_LA + w[lz_la]*phi_LZ*now[9];
                             delta[7] := plus - minus*now[7];

                             F.AlgalSinking := F.AlgalSinking + ActSink_SA*now[7];

                             plus := AE[sz_sa]*w[sz_sa]*phi_SZ*now[6];
                             minus := death_SZ;   { check this construction out w/Steve }
                             delta[8] := (plus - minus) * now[8];

                              F.ZoopDeathInMeta := death_sz*now[8];
                              F.ZoopCons := F.ZoopCons + w[sz_sa]*phi_SZ*now[6]*now[8];

                             IF (MetaMigLocations.LZhere=1.0) THEN
                                  BEGIN
                                  plus := (AE[lz_sa]*w[lz_sa]*now[6] + AE[lz_la]*w[lz_la]*now[7])*phi_LZ;
                                  minus := death_LZ;   { check this construction out w/Steve }

                                  F.ZoopDeathInMeta := F.ZoopDeathInMeta + death_LZ*now[9];
                                  F.ZoopCons := F.ZoopCons + (w[lz_sa]*now[6] + w[lz_la]*now[7])*phi_SZ*now[8];

                                  delta[9] := (plus - minus) * now[9];
                                  END;
                             IF (MetaMigLocations.LZhere=0.0) AND (EpiMigLocations.LZhere=0.0) THEN
                                  delta[9] := 0.0;
                             END;
        END
END;

{ ************************************************************* }

PROCEDURE MakeAverageRatesAndAddToTotal (F1,F2,F3,F4 : FluxRec; VAR TotalF : FluxRec);

{ Averages fluxes over the four calls to Runge-Kutta }
{ last modified 11 May 1992 }

BEGIN
     WITH TotalF DO
          BEGIN
          ZoopCons := ZoopCons + 0.25 * (F1.ZoopCons + F2.ZoopCons + F3.ZoopCons + F4.ZoopCons);
          ChaobCons := ChaobCons + 0.25 * (F1.ChaobCons + F2.ChaobCons + F3.ChaobCons + F4.ChaobCons);
          SFConsofZoops := SFConsofZoops + 0.25 * (F1.SFConsofZoops +F2.SFConsofZoops +F3.SFConsofZoops +F4.SFConsofZoops);
          SFConsofChaob := SFConsofChaob + 0.25 * (F1.SFConsofChaob +F2.SFConsofChaob +F3.SFConsofChaob +F4.SFConsofChaob);
          SFConsofLittoral := SFConsofLittoral + 0.25 * (F1.SFConsofLittoral + F2.SFConsofLittoral
                           + F3.SFConsofLittoral + F4.SFConsofLittoral);
          LFConsofSF := LFConsofSF + 0.25 * (F1.LFConsofSF +F2.LFConsofSF +F3.LFConsofSF +F4.LFConsofSF);
          LFConsofChaob := LFConsofChaob + 0.25 * (F1.LFConsofChaob +F2.LFConsofChaob +F3.LFConsofChaob +F4.LFConsofChaob);
          LFConsofLittoral := LFConsofLittoral + 0.25 * (F1.LFConsofLittoral + F2.LFConsofLittoral
                           + F3.LFConsofLittoral + F4.LFConsofLittoral);
          ZoopExcr := ZoopExcr + 0.25 * (F1.ZoopExcr + F2.ZoopExcr + F3.ZoopExcr + F4.ZoopExcr);
          ChaobExcr := ChaobExcr + 0.25 * (F1.ChaobExcr + F2.ChaobExcr + F3.ChaobExcr + F4.ChaobExcr);
          SFExcr := SFExcr + 0.25 * (F1.SFExcr + F2.SFExcr +F3.SFExcr +F4.SFExcr);
          LFExcr :=  LFExcr + 0.25 * (F1.LFExcr + F2.LFExcr +F3.LFExcr +F4.LFExcr);
          AlgalSinking := AlgalSinking + 0.25 * (F1.AlgalSinking + F2.AlgalSinking + F3.AlgalSinking + F4.AlgalSinking);
          AlgalPUptake := AlgalPUptake + 0.25 * (F1.AlgalPUptake + F2.AlgalPUptake+ F3.AlgalPUptake + F4.AlgalPUptake);
          END;
END;

{ ************************************************************* }

PROCEDURE rk ( TimeStep : integer;
               y : SVvector;
               VAR yout : SVvector;
               VAR AverageFluxes : FluxRec );

{Last modified 27 April 1992 }

{Uses Runge-Kutta Order 4 integration approximations to "solve" the system of differential equations.
y is a vector of current state variables, yout is the vector of computed derivatives. }

{Accesses ... }

CONST
        nderivs = 12;   {Number of derivatives in derivs procedure}
VAR
        h, hh, h6 : REAL;
        yt, dydx, dym, dyt : SVvector;
        i : INTEGER;
        F1,F2,F3,F4 : FluxRec;
BEGIN
        h := 1/NumberOfStepsPerDay;
        hh := h * 0.5;
        h6 := h / 6;

        ZeroFluxRec(F1);
        ZeroFluxRec(F2);
        ZeroFluxRec(F3);
        ZeroFluxRec(F4);

        derivs(y, TimeStep, dydx, F1);
        for i := 1 to nderivs do
                yt[i] := y[i] + hh * dydx[i];

        derivs(yt, TimeStep, dyt, F2);
        for i := 1 to nderivs do
                yt[i] := y[i] + hh * dyt[i];

        derivs(yt, TimeStep, dym, F3);
        for i := 1 to nderivs do
                BEGIN
                yt[i] := y[i] + h * dym[i];
                dym[i] := dyt[i] + dym[i];
                END;

        derivs(yt, TimeStep, dyt, F4);
        for i := 1 to nderivs do
                yout[i] := y[i] + h6 * (dydx[i] + dyt[i] + 2.0 * dym[i]);

        MakeAverageRatesAndAddToTotal(F1,F2,F3,F4,SummerFluxes);

END;    {procedure rk}

{ ********************************************************************** }

PROCEDURE SaveStateVariables (counter, {array index}
                         DayNumber:INTEGER; {what day number this is}
                         y: SVvector); {current values}

{from y, the record of current state variable values, to the archival record of values}

{last modified 13 January 1992}

BEGIN
      WITH StateVars DO
            BEGIN
                 day[counter] := DayNumber;
                 WITH epilimnion DO
                         BEGIN 
                         P[counter] := y[1];
                         SA[counter] := y[2];     LA[counter] := y[3];
                         SZ[counter] := y[4];     CH[counter] := y[10];    
                         SF[counter] := y[11];    LF[counter] := y[12];
                         END;
                 WITH metalimnion DO
                         BEGIN
                         P[counter] := y[5];           SA[counter] := y[6];
                         LA[counter] := y[7];          SZ[counter] := y[8];
                         END;
                 LZ[counter] := y[9];
             END;  {with state variables do}
END;

{ *************************************************************************** }

PROCEDURE CheckFlags (SmAlgae, LarAlgae : REAL; isthisepi : BOOLEAN);

{ last modified 3 January 1992 }
{ Sets signal flags for areas of interest; basically lifted directly from SRC's BG Mendota model}
{ Called once for the Epilimnion, once for the metalimnion }
{ Changes the flags by side effect; accesses SBP.ChltoP ratios globally }

VAR
        SACheck, LACheck, BloomCheck : REAL; 
        Bloom, LAdom, Clear : BOOLEAN;
BEGIN
        LACheck := LarAlgae * SBP.ChltoP_LA;
        SACheck := SmAlgae * SBP.ChltoP_SA;
        BloomCheck := LACheck + SACheck;    
        LAdom := false;  Bloom := false;  Clear := false;

        IF (LACheck >= 4.0*SACheck) THEN
                 LAdom := true;
        IF LAdom AND (BloomCheck>=30.0) THEN
                 Bloom := true;
        IF (BloomCheck <= 10.0) THEN
                 Clear := true;

        IF isthisepi THEN
                 BEGIN
                 IF LAdom THEN NDaysLADominEpi := NDaysLADominEpi + 1;
                 IF Clear THEN NClearDaysinEpi := NClearDaysinEpi + 1;
                 IF Bloom THEN NBloomDaysinEpi := NBloomDaysinEpi + 1;
                 END
        ELSE
                 BEGIN
                 IF LAdom THEN NDaysLADominMeta := NDaysLADominMeta + 1;
                 IF Clear THEN NClearDaysinMeta := NClearDaysinMeta + 1;
                 IF Bloom THEN NBloomDaysinMeta := NBloomDaysinMeta + 1;
                 END
END;

{ ************************************************************************* }

PROCEDURE Driver ( CurrPInput : REAL; WhenToPerturb : INTEGER );
{ Runs the guts of the simulation!}
{ Accesses NumberofWeeksToRun, UsersWeekSize, SaveInterval globally;
  accesses and changes the 6 flags (NDaysLADom, NClearDays, NBloomDays) globally }
{ Changes coefficient records, state variables by side effect }

{ last modified 11 March 1993 }
{ added pulse perturbation on day = "When to Perturb" }

VAR
        DayNumber, WeekNumber, SVindex, k,
        SimulationCounter, WithinWeekCounter, WithinDayCounter, l : integer;    {loop counters}
        VectorIntoRK, VectorOutofRK : SVvector;
        AvgFlux : FluxRec;
BEGIN
        ConvertFromStateVariableRecordtoVector (1, VectorIntoRK);

        { * initialize variables * }
        DayNumber := 1;             SVindex := 2;
        StateVars.day[1] := 1.0;    WeekNumber := 2;

        { * Initialize flags * }
        NDaysLADominEpi := 0;     NClearDaysinEpi := 0;             NBloomDaysinEpi := 0;
        NDaysLADominMeta := 0;    NClearDaysinMeta := 0;            NBloomDaysinMeta := 0;

        clrscr; gotoxy(25,10); writeln('Now simulating ');

        { * Simulate * }
        FOR SimulationCounter:=1 TO NumberofWeekstoRun DO
                BEGIN

                gotoxy(35,15);   writeln('Week Number ',SimulationCounter);
                gotoxy(35,16);   writeln('Nutrient Loading Level = ', CurrPInput:5:2);
                gotoxy(35,17);   IF StateVars.epilimnion.LF[1]=0.0 THEN
                                       writeln('Planktivore-Dominated')
                                 ELSE  writeln('Piscivore-Dominated');
                gotoxy(35,18);   IF PulseofNutrients<>0.0 THEN
                                       writeln('Perturbed with ', PulseofNutrients:5:2, ' ug/L on day ',WhenToPerturb:3)
                                 ELSE  writeln('Unperturbed');

                FOR WithinWeekCounter := 1 to WeekSize DO
                        BEGIN
                        DayNumber := DayNumber + 1;
                        IF (DayNumber mod WeekSize)=0 THEN  {go out to the "field" and sample (get a new input schedule)}
                                BEGIN
                                GetThisWeeksSched(WeekNumber);
                                WeekNumber := WeekNumber + 1;
                                END;

                        IF (DayNumber = WhenToPerturb) THEN
                                BEGIN
                                VectorIntoRK[1] := VectorIntoRK[1] + PulseofNutrients;
                                gotoxy(35,19);
                                writeln('Pulse has been Sent');
                                END;

                        {*Get today's epilimnetic light Schedules}
                        epi.phys.Ir := DailySch[DayNumber,1];
                        epi.phys.Pinput := CurrPInput;

                        FOR WithinDayCounter := 1 to NumberofStepsPerDay do
                                 BEGIN
                                 rk(WithinDayCounter, VectorIntoRK, VectorOutofRK, SummerFluxes);

                                 FOR l := 1 to NStateVars DO
                                         BEGIN
                                         VectorIntoRK[l]:= VectorOutofRK[l];
                                         IF VectorIntoRK[l] <= 0 THEN
                                                 BEGIN
                                                 VectorIntoRK[l] := 0.000001;
                                                { writeln('Variable ', l, ' got too low');}
                                                 END;
                                         END;
                                 END;  {1 time step}

                        CheckFlags(VectorIntoRK[2], VectorIntoRK[3], true);
                        CheckFlags(VectorIntoRK[6], VectorIntoRK[7], false);     

                        IF (DayNumber mod SaveInterval) = 0 THEN
                                 BEGIN
                                 SaveStateVariables(SVindex, Daynumber, VectorIntoRK);
                                 SVindex := SVindex + 1;
                                 END;

                        END;  {1 day}
                END;  {1 week}
END;

{ ******************************************************************** }

PROCEDURE PrintFlagResults ( VAR outfile : TEXT );

{ last modified 3 January 1992 }
{ Prints final values of the flags to outfile }

BEGIN
        writeln(outfile);
        writeln(outfile, 'Epilimnetic Flag Results:');
        writeln(outfile, '   ', NDaysLADominEpi, ' days of inedible algae domination');
        writeln(outfile, '   ', NBloomDaysinEpi, ' bloom days');
        writeln(outfile, '   ', NClearDaysinEpi, ' clear days');

        writeln(outfile);
        writeln(outfile, 'Metalimnetic Flag Results:');
        writeln(outfile, '   ', NDaysLADominMeta, ' days of inedible algae domination');
        writeln(outfile, '   ', NBloomDaysinMeta, ' bloom days');
        writeln(outfile, '   ', NClearDaysinMeta, ' clear days');

        writeln(outfile);
        IF PrintDefinitions THEN
                 BEGIN
                 writeln(outfile, '(Inedible Algae "dominate" when LA chl >= 4 * SA chl)');
                 writeln(outfile, '(Blooms occur when inedible algae dominate and total chl >= 30 ug/l)');
                 writeln(outfile, '(Clear Days occur when total chl is <= 10 ug/l)');
                 writeln(outfile)
                 END
END;

{ ******************************************************************** }

PROCEDURE PrintSimulationValuestoASCIIFile ( VAR outfile : TEXT; Perturbed : REAL );

{ last modified 8 February 1992 }
{ prints actual values for the simulations to a file for import into SYSTAT.}

VAR
        i, LastValue : INTEGER;

BEGIN
        LastValue := (NumberofWeekstoRun * WeekSize) div SaveInterval;

        WITH StateVars DO
                 FOR i := 1 to LastValue DO
                        BEGIN
                        write(outfile, day[i]:5:0,' ',epi.phys.PInput:4:2,' ',PulseofNutrients:5:3,' ');

                        WITH epilimnion DO
                             write(outfile, day[i]:4:0,' ', P[i]:10:5, ' ', SA[i]:10:5, ' ',  LA[i]:10:5, ' ', SZ[i]:10:5);
                        WITH metalimnion DO
                             write(outfile, ' ', P[i]:10:5, ' ', SA[i]:10:5, ' ', LA[i]:10:5, ' ', SZ[i]:10:5);
                        WITH epilimnion DO
                             write(outfile, ' ', LZ[i]:10:5, ' ', CH[i]:10:5, ' ', SF[i]:10:5, ' ', LF[i]:10:5);
                        writeln(outfile);
                        END;

END;

{ ************************************************************* }

PROCEDURE AggregateVariables;
{ Last modified 9 December 1993 (changed how aggregation occurs) }
VAR
        epithick, metathick, totalthick, dum1, dum2 : real;
        last, i, week: integer;
BEGIN
        week := 0;
        last := NumberofWeekstoRun*WeekSize;

        WITH StateVars DO
        WITH SummaryVars DO
             FOR i := 1 to Last DO
                 BEGIN
                 { determine thickness for this period }
                 IF (i mod 7)=1 THEN  {it's a new week}
                      BEGIN
                      week := week + 1;
                      epithick := WeeklySch[week].EpiThick;
                      metathick := WeeklySch[week].MetaThick;
                      totalthick := epithick + metathick;
                      END;

                 { Phosphorus Compartment }
                      Phosphorus[i] := (epilimnion.P[i]*epithick + metalimnion.P[i]*metathick) / totalthick;

                 { Algal Compartment }
                      dum1 := epilimnion.SA[i] + epilimnion.LA[i];
                      dum2 := metalimnion.SA[i] + metalimnion.LA[i];
                      Algae[i] := (dum1*epithick + dum2*metathick) / totalthick;

                 { Zooplankton Compartment }
                      dum1 := epilimnion.SZ[i] + LZ[i];
                      dum2 := metalimnion.SZ[i];
                      Zooplankton[i] := (dum1*epithick + dum2*metathick) / totalthick;

                 { Chaoborus Compartment }
                      Chaoborus[i] := epilimnion.CH[i];

                 { Small Fish Compartment }
                      SmallFish[i] := epilimnion.SF[i];

                 { Large Fish Compartment }
                     LargeFish[i] := epilimnion.LF[i];

                 END;
END;

{ ************************************************************* }

PROCEDURE PrintAggregatedVariablesToAsciiFile (VAR Outfile : TEXT);
{ last modified 9 Decmeber 1993 }
VAR
     i, last : INTEGER;

BEGIN
     last := NumberofWeeksToRun*Weeksize;

     WITH SummaryVars DO
          FOR i := 1 to last DO
                writeln(Outfile,StateVars.day[i]:5:0,' ',epi.phys.Pinput:5:3,' ',PulseofNutrients:5:3,' ',
                        Phosphorus[i]:9:5, ' ', Algae[i]:9:5, ' ',
                        Zooplankton[i]:9:5, ' ', Chaoborus[i]:9:5, ' ',
                        SmallFish[i]:9:5, ' ', LargeFish[i]:9:5);
END;

{ ****************************************************************** }

PROCEDURE CalculateAggregatedMeansAndMaxes (VAR Mean, Maxes : SixVector);
{ Last modified 9 December 1993 (changed how aggregation occurs) }
{ Computes the means and max sizes for the six compartments in the linear model }
{ turned off epi-only calculations }
VAR
        epithick, metathick, totalthick, dum1, dum2,
        sum, max : real;
        last, i : integer;
BEGIN
        last := NumberofWeekstoRun*WeekSize;

        WITH SummaryVars DO
             BEGIN
             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + Phosphorus[i];
                    if (Phosphorus[i] > max) then max := Phosphorus[i];
                    END;
             mean[1] := sum / last;
             maxes[1] := max;

             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + Algae[i];
                    if (Algae[i] > max) then max := Algae[i];
                    END;
             mean[2] := sum / last;
             maxes[2] := max;

             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + Zooplankton[i];
                    if (Zooplankton[i] > max) then max := Zooplankton[i];
                    END;
             mean[3] := sum / last;
             maxes[3] := max;

             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + Chaoborus[i];
                    if (Chaoborus[i] > max) then max := Chaoborus[i];
                    END;
             mean[4] := sum / last;
             maxes[4] := max;

             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + SmallFish[i];
                    if (SmallFish[i] > max) then max := SmallFish[i];
                    END;
             mean[5] := sum / last;
             maxes[5] := max;

             sum := 0;   max := 0;
             FOR i := 1 TO last DO
                    BEGIN
                    sum := sum + LargeFish[i];
                    if (LargeFish[i] > max) then max := LargeFish[i];
                    END;
             mean[6] := sum / last;
             maxes[6] := max;

             END;
END;

{ ************************************************************* }

PROCEDURE SystemNutrientTurnoverRates (VAR A : Matrix; means : SixVector;
          VAR StandingStock : Real);
{ calculates input-based and output-based ntr calculations }
{ last modified 9 December 1993 }

VAR
     inputs, outputs, NTR : REAL;
     i : INTEGER;
BEGIN
     inputs := 0.0;
     FOR i := 1 to MatrixSize DO
         inputs := inputs + a[i,0]*means[i];

     outputs := 0.0;
     FOR i := 1 to MatrixSize DO
         outputs := outputs + a[0,i]*means[i];

     StandingStock := 0.0;
     FOR i := 1 to MatrixSize DO
         StandingStock := StandingStock + means[i];

     NutTurnRates[SimNumber,1] := inputs/StandingStock;
     NutTurnRates[SimNumber,2] := outputs/StandingStock;

END;

{ ************************************************************* }

PROCEDURE CompartmentNutrientTurnoverRates (VAR A : Matrix; means : SixVector);
{ calculates input-based and output-based ntr calculations for each compartment }
{ last modified 9 December 1993 }

VAR
     InputSum, OutputSum, StandingStock, NTR : REAL;
     i, j : INTEGER;
BEGIN
     FOR i := 1 to MatrixSize DO
         BEGIN
         InputSum := 0.0;
         OutputSum := 0.0;
         FOR j := 0 to MatrixSize DO
             IF i<>j THEN
                  BEGIN
                  InputSum := InputSum + a[i,j]*means[j];
                  OutputSum := OutputSum + a[j,i]*means[i];
                  END;

         ComptTurnRates[PerturbCounter,i,1] := InputSum / means[i];
         ComptTurnRates[PerturbCounter,i,2] := OutputSum / means[i];

         END;
END;

{ ************************************************************* }

PROCEDURE PrintNutrientTurnoverRates (VAR outfile : TEXT;
          Means : SixVector; StandingStock : REAL);
{ last modified 9 December 1993 }

VAR
   i,j : INTEGER;

BEGIN
     writeln(outfile,epi.phys.Pinput:5:3,' ',PulseofNutrients:5:3,' ',
        0:3,' ',NutTurnRates[PerturbCounter,1]:10:5, ' ', NutTurnRates[PerturbCounter,2]:10:5,' ',
        StandingStock:10:5);
     FOR j := 1 to MatrixSize DO
         BEGIN
         writeln(outfile,epi.phys.Pinput:5:3,' ', PulseofNutrients:5:3,' ',
             j:3,' ',ComptTurnRates[PerturbCounter,j,1]:10:5,' ', ComptTurnRates[PerturbCounter,j,2]:10:5,' ',
             Means[j]:10:5);
         END;
END;

{ ************************************************************* }

PROCEDURE LinearMatrixCalculations (VAR A : Matrix; VAR Means : SixVector);
{ does guts of calculations for linear model }
{ last modified 9 December 1993 :
   (1) each internal transfer rate divided by size of the DONOR compartment
   (2) all input rates in ug/L/d not /d, so don't need benthic compartment size
   (3) fudged external sources, which aren't included in the model,
       similarly to how the empirical calculations are done:
       1% of maximum observed value / (mean value*number of days in simulation)
   (4) turned diagonals back on
}

{ previously examined 8 February 1993; no changes made }

VAR
     Maxes : SixVector;
     Last : integer;
     deltaT : REAL;
BEGIN
     Last := (NumberofWeekstoRun * WeekSize) div SaveInterval;
     deltaT := 1.0 * Last * NumberofStepsPerDay;
     CalculateAggregatedMeansandMaxes(Means,Maxes);

   WITH SummerFluxes DO
     BEGIN
     a[1,0] := epi.phys.Pinput;
     a[1,2] := 0.0;
     a[1,3] := ZoopExcr / (means[3] * deltaT);
     a[1,4] := ChaobExcr / (means[4] * deltaT);
     a[1,5] := SFExcr / (means[5] * deltaT);

     a[2,0] := 0.01 * Maxes[2] / (Last);
     a[2,1] := AlgalPUptake / (means[1] * deltaT);
         IF a[2,1]>1 THEN
            a[2,1] := 0.95;  { If P uptake is too rapid, cut to 0.95 }
     a[2,3] := 0.0;
     a[2,4] := 0.0;
     a[2,5] := 0.0;

     a[3,0] := 0.01 * Maxes[3] / (Last);
     a[3,1] := 0.0;
     a[3,2] := ZoopCons / (means[2] * deltaT);
     a[3,4] := 0.0;
     a[3,5] := 0.0;

     a[4,0] := 0.01 * Maxes[4] / (Last);
     a[4,1] := 0.0;
     a[4,2] := 0.0;
     a[4,3] := ChaobCons / (means[3] * deltaT);
     a[4,5] := 0.0;

     a[5,0] := SFConsofLittoral / deltaT;
     a[5,1] := 0.0;
     a[5,2] := 0.0;
     a[5,3] := SfConsofZoops / (means[3] * deltaT);
     a[5,4] := SFConsofChaob / (means[4] * deltaT);

     a[0,1] := epi.phys.flush;
     a[0,2] := AlgalSinking / (means[2] * deltaT);
     a[0,3] := a[1,3] + ZoopDeathInMeta / (means[3] * deltaT);
     a[0,4] := (0.33 * ChaobCons) / (means[4] * deltaT);
     a[0,5] := SBP.death_SF + egest_SF * (SFConsofZoops+SFConsofChaob+SFConsofLittoral) / (means[5] * deltaT);

     a[0,6] := SBP.death_LF + egest_LF * (LFConsofSF+LFConsofChaob+LFConsofLittoral) / (means[6] * deltaT);
     a[1,6] := LFExcr / (means[6] * deltaT);
     a[2,6] := 0.0;
     a[3,6] := 0.0;
     a[4,6] := 0.0;
     a[5,6] := 0.0;

     a[6,0] := LFConsofLittoral / (deltaT);
     a[6,1] := 0.0;
     a[6,2] := 0.0;
     a[6,3] := 0.0;
     a[6,4] := LFConsofChaob / (means[4] * deltaT);
     a[6,5] := LFConsofSF / (means[5] * deltaT);

     a[1,1] :=  -1*(a[0,1] + a[2,1] + a[3,1] + a[4,1] + a[5,1]  + a[6,1]);
     a[2,2] :=  -1*(a[0,2] + a[1,2] + a[3,2] + a[4,2] + a[5,2]  + a[6,2]);
     a[3,3] :=  -1*(a[0,3] + a[1,3] + a[2,3] + a[4,3] + a[5,3]  + a[6,3]);
     a[4,4] :=  -1*(a[0,4] + a[1,4] + a[2,4] + a[3,4] + a[5,4]  + a[6,4]);
     a[5,5] :=  -1*(a[0,5] + a[1,5] + a[2,5] + a[3,5] + a[4,5]  + a[6,5]);
     a[6,6] :=  -1*(a[0,6] + a[1,6] + a[2,6] + a[3,6] + a[4,6]  + a[5,6]);

     END;
END;

{ ************************************************************* }

PROCEDURE CloseUpThisOne ( Perturbed : REAL);
{ last modified 9 December 1993 }
VAR
        Means : SixVector;  SS : Real;
BEGIN
      { MakeSummaryRecord; }
      { StatisticalSummary(SimulationResults, Perturbed); }
      { PrintFlagResults(SimulationResults); }
      { PrintSimulationValues(SimulationResults, Perturbed); }
      { PrintOutputLine(SimNumber, MeanResults, Perturbed); }
        PrintSimulationValuestoASCIIfile(SysOutfile1, Perturbed);
        InitializeMatrix(SummerMatrix);
        AggregateVariables;
        PrintAggregatedVariablesToAsciiFile(SysOutfile2);
        LinearMatrixCalculations(SummerMatrix, Means);
        PrintFluxes(SummerMatrix, FluxFile);
        PrintMatrix(SummerMatrix, MatrixFile, InputVectorFile);
        SystemNutrientTurnoverRates(SummerMatrix, Means, SS);
        CompartmentNutrientTurnoverRates(SummerMatrix, Means);
        PrintNutrientTurnoverRates(NutrRates, Means, SS);
END;

{ ************************************************************* }

PROCEDURE CloseUpThisSet;
{last modified 9 December 1993 specifically for the linear version}
BEGIN
    close(MatrixFile);
    close(FluxFile);
    close(InputVectorFile);
    close(SysOutfile1);
    close(SysOutfile2);
END;

{ ************************************************************* }
 
BEGIN { ** MAIN! last modified 6 April ** }

        continue := true;   SimNumber := 1;

        assign(userinfile, 'sysinput.dat');
        StartUpProgram;

        CurrentNutrLoad := NutrLoadings[1];

        FOR PerturbCounter := 1 to NPerts DO
              BEGIN
              StartUpNewSet;

              PulseofNutrients := PerturbArray[PerturbCounter];

              StartUpNewRun;
              Driver(CurrentNutrLoad, PerturbNumber);
              CloseUpThisOne(0.0);
              CloseUpThisSet;

              SimNumber := SimNumber + 1;

              END;

        Close(nutrrates);

END.
